/* (C) Universitaet Passau 1986-1991 */
%{/************************************************************************/
/*									*/
/*				stdparser.y				*/
/*									*/
/************************************************************************/


/* Stacksize for yacc	*/

#define	YYMAXDEPTH 2000

extern int yylex();
extern int yyerror(char *);

#include <LEDA/graph.h>
#include <Tim/GA.h>

// The Graph to load ...
extern	GAGraph   *parsed_graph;
extern  KeyMapper keymapper;

// The current's
graph*  current_graph;
node	current_node;
edge	current_edge;
GAList*	current_attrs;
GA*	current_attr;

list<GA*>	attribute_stack;
list<GAList*>	attributes_stack;

// Find a node by its number
static node find (GAGraph& g, int nr);

KeyType nr_key;
KeyType label_key;

%}
%union	{	/* stack for parser	*/
	int	nr;
	float	floatnr;
	char	*text;
	char	*key;
}
	
%token			SCANNER_GRAPH '=' ';' SCANNER_END SCANNER_GRAPHS
%token			SCANNER_DIRECTED SCANNER_UNDIRECTED
%token			SCANNER_BEGIN_GRAPHED_ATTRIBUTES
%token			SCANNER_END_GRAPHED_ATTRIBUTES
%token	<nr>		SCANNER_NUMBER
%token	<floatnr>	SCANNER_FLOATNUMBER
%token	<text>		SCANNER_IDENTIFIER
%token	<key>		SCANNER_KEY

%type	<text>	label nodelabel edgelabel
%type	<nr>	node_nr targetnode_nr

%start		parser_start
%%
parser_start :		  graph
			| error
			{
				/* Syntax Error	*/
				YYABORT;
			}
			;


graph :			{
				nr_key = keymapper.add ("NR:");
				label_key = keymapper.add ("LABEL:");
			}
			SCANNER_GRAPH label optional_equal
			{
				current_graph = parsed_graph->get();
				if (current_graph == NULL) {
					current_graph = new graph;
					parsed_graph->set (current_graph);
				}
				current_attrs = new GAList();
			}
			directedness
			{
				/* momentan nur directed */
			}
			graph_attributes
			{
				parsed_graph->graph_attrs() = current_attrs;
			}
			list_of_nodes
			SCANNER_END
			{
			}
			;

label :			SCANNER_IDENTIFIER
			{
				$$ = $1;
			}
			|
			{
				$$ = NULL;
			}
			;

optional_equal :	 | '='
			;

directedness :		  /* nothing or ... */
			{
			}
			| SCANNER_DIRECTED
			{
				current_graph->make_directed ();
			}
			| SCANNER_UNDIRECTED
			{
				current_graph->make_undirected ();
			}
			;

graph_attributes :	/* nothing */
			|
			SCANNER_BEGIN_GRAPHED_ATTRIBUTES
			attributes
			SCANNER_END_GRAPHED_ATTRIBUTES
			;


list_of_nodes :		  /* nothing or ... */
			| node list_of_nodes
			;

node :			node_nr
			{
				GA* nr = new GA(nr_key);
				nr->append (new GAValue($1));

			        current_node = find(*parsed_graph, $1);
				current_attrs = new GAList();
				current_attrs->append(nr);
			}
			node_attributes
			nodelabel
			{
				parsed_graph->node_attrs(current_node) =
					current_attrs;
				if ($4 != NULL) {
					GA* label = new GA (label_key);
					label->append (new GAValue($4));
					current_attrs->append(label);
				}
			}
			list_of_edges
			;

node_nr :		SCANNER_NUMBER
			{ $$ = $1; }
			;

nodelabel :		label
			{ $$ = $1; }
			;

node_attributes :	/* nothing */
			|
			SCANNER_BEGIN_GRAPHED_ATTRIBUTES
			attributes
			SCANNER_END_GRAPHED_ATTRIBUTES
			;


list_of_edges :		 ';'
			| edge list_of_edges
			;

edge :			targetnode_nr
			{
				current_edge = current_graph->new_edge (
					current_node,
					find(*parsed_graph, $1));
				parsed_graph->edge_attrs(current_edge) = new GAList();
				current_attrs = parsed_graph->edge_attrs(current_edge);
			}
			edge_attributes
			edgelabel
			{
				if ($4 != NULL) {
					GA* label = new GA (label_key);
					label->append (new GAValue($4));
					current_attrs->append(label);
				}
			}
			;

targetnode_nr :		SCANNER_NUMBER
			{ $$ = $1; }
			;

edgelabel :		label
			{
			}
			;

edge_attributes :	/* nothing */
			|
			SCANNER_BEGIN_GRAPHED_ATTRIBUTES
			attributes
			SCANNER_END_GRAPHED_ATTRIBUTES
			;


attribute :		SCANNER_KEY
			{
				current_attr = new GA(keymapper.add($1));
				current_attrs->append(current_attr);
			}
			attribute_values
			{
			}
			| ',' | ';'
			/* a little hack to cope with old file formats */
			;

attributes :		  /* nothing */
			| attribute attributes
			;

attribute_values:	  /* nothing */
			| attribute_value attribute_values
			;

attribute_value :	  SCANNER_NUMBER
			{
				current_attr->append (new GAValue($1));
			}
			| SCANNER_IDENTIFIER
			{
				current_attr->append (new GAValue($1));
			
			}
			|
			'['
			{
				attribute_stack.push(current_attr);
				attributes_stack.push(current_attrs);
				current_attrs = new GAList();
			}
			attributes
			{
				attribute_stack.head()->append(
					new GAValue(current_attrs));
				current_attrs = attributes_stack.pop();
				current_attr = attribute_stack.pop();
			}
			']'
			;
%%
#include <iostream.h>

int     yyerror (char *str)
{
        cout << "ERROR: " << str << "\n";
	return 42;
}

static node find (GAGraph& g, int nr)
{
    node v;
    
    forall_nodes (v, *current_graph) {
 	if (g.number(v) == nr) {
	    return v;
	}
    }
    
    v = current_graph->new_node();
    g.set_number(v,nr);
    return v;
}
